#include "randomquotes.h"
#include <QGridLayout>
#include <QMessageBox>
#include <stdlib.h>
#include <time.h>

//constructor
RandomQuotes::RandomQuotes(){
    //initializes the timer
    timer = new QTimer(this);    
    //sets up the GUI
    setUpGUI();
}

//sets up the GUI and connects signals and slots
void RandomQuotes::setUpGUI(){
    //initializes all the widgets
    timerLabel = new QLabel("Set Timer [Range: 5000 - 20000 ms] ");
    timerEdit = new QLineEdit();
    quoteEdit = new QTextEdit();
    okButton = new QPushButton("Set Timer");
    cancelButton = new QPushButton("Cancel");

    //QTextEdit is made read-only to prohibit user from entering data
    quoteEdit->setReadOnly(true);

    // creates a layout and places widgets on the layout
    QGridLayout* layout = new QGridLayout();
    layout->addWidget(timerLabel, 0,0);
    layout->addWidget(timerEdit, 0,1);
    layout->addWidget(okButton, 1,1);
    layout->addWidget(quoteEdit, 2,1);
    layout->addWidget(cancelButton,3,1 );    
    //layout is set to the root widget
    this->setLayout(layout);    
    //title of the root widget is set
    this->setWindowTitle("Random Quotes");

    //signals and slots are connected
    connect(okButton,SIGNAL(clicked()), this, SLOT(setTimer()));
    connect(cancelButton, SIGNAL(clicked()), this, SLOT(cancelTimer()));
    connect(timer, SIGNAL(timeout()),this, SLOT(displayQuotes()));
}

//checks if the user input is valid and starts the timer
void RandomQuotes::setTimer(){
     bool ok;
     QString timerText = timerEdit->text();
     int timerValue = timerText.toInt(&ok, 10);

     if (ok && (timerValue >= 5000) && (timerValue <= 20000)){
         timer->setInterval(timerValue);
         timer->start();
         timerEdit->setReadOnly(true);         
         okButton->setDown(true);
     }
     else{
         QMessageBox:: information(this, "Invalid timer value", "Invalid or out of range timer value.");
     }
}

//quotes are randomnly selected to be displayed when the timer emits timeout() signal
void RandomQuotes::displayQuotes(){
    quoteEdit->clear();    
    quoteEdit->setText(qf.getAQuote());
}

//stops the timer and allows user to reset the timer
void RandomQuotes::cancelTimer(){
    timer->stop();
    quoteEdit->clear();
    okButton->setDown(false);
    timerEdit->clear();
    timerEdit->setReadOnly(false);
}

