#include "webbrowser.h"
#include <QLayout>
#include <QToolButton>
#include <QLineEdit>
#include <QWebView>
#include <QWebFrame>
#include <QWebElementCollection>
#include <QSplitter>
#include <QStringListModel>

WebBrowser::WebBrowser(QWidget *parent) :
    QWidget(parent)
{
    addressBar = new QLineEdit(this);
    backButton = new QToolButton(this);
    forwardButton = new QToolButton(this);
    reloadButton = new QToolButton(this);
    findLinksButton = new QPushButton("Display Links",this);
    findtext = new QLineEdit(this);
    findtext->setText("type search string and press Enter");

    browser = new QWebView(this);

    backButton->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
    forwardButton->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
    reloadButton->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);

    QHBoxLayout *toolsLayout = new QHBoxLayout;    
    toolsLayout->addWidget(findtext);
    toolsLayout->addWidget(backButton);
    toolsLayout->addWidget(forwardButton);
    toolsLayout->addWidget(reloadButton);
    toolsLayout->addWidget(addressBar);
    toolsLayout->addWidget(findLinksButton);

    QVBoxLayout *mainLayout = new QVBoxLayout(this);
    mainLayout->addLayout(toolsLayout);

    linkList = new QListView();
    QSplitter *splitter = new QSplitter;
    splitter->addWidget(browser);
    splitter->addWidget(linkList);
    mainLayout->addWidget(splitter);


    backButton->setDefaultAction(browser->pageAction(QWebPage::Back));
    forwardButton->setDefaultAction(browser->pageAction(QWebPage::Forward));
    reloadButton->setDefaultAction(browser->pageAction(QWebPage::Reload));  

    connect(addressBar, SIGNAL(returnPressed()), this, SLOT(loadPage()));
    connect(browser, SIGNAL(urlChanged(QUrl) ), this, SLOT(updateAddressBar(QUrl) ));
    connect(findtext, SIGNAL(returnPressed()), this, SLOT(highlight()));
    connect(findLinksButton, SIGNAL(clicked()), this,SLOT(listLinks()));
    connect(browser,SIGNAL(linkClicked(QUrl)), this, SLOT(addURL(QUrl)));

    linkList->setModel(&model);
}

void WebBrowser::loadPage() { 
    browser->load(QUrl::fromUserInput(addressBar->text()));
    browser->page()->setLinkDelegationPolicy(QWebPage::DelegateAllLinks);

}

void WebBrowser::updateAddressBar(const QUrl &url) {
    addressBar->setText(url.toString());
}

void WebBrowser::highlight(){   
    browser->page()->findText(findtext->text(),QWebPage::HighlightAllOccurrences);
}

void WebBrowser::listLinks(){

    links.clear();
    QWebFrame* frame = browser->page()->mainFrame();
    QWebElementCollection elements = frame->findAllElements("a");

    foreach(QWebElement we, elements){
          QString src = we.attribute("href");          
          if (!src.isEmpty() && !(we.toPlainText().trimmed().isEmpty()))
            links.append(we.toPlainText().trimmed());
       }
    model.setStringList(links);
}


void WebBrowser::addURL(const QUrl &url){
    browser->load(url);    
}


