#include <QMenu>
#include <QMenuBar>
#include <QTextEdit>
#include <QToolBar>
#include <QTextCursor>

#include "mynotepad.h"
#include "textandstylememento.h"

//constructor
MyNotepad::MyNotepad(): textEdit(new QTextEdit) { //initialize data member
    //set title of the window
    setWindowTitle("MyNotepad");  

    //initialize actionGroup
    actionGroup = new QActionGroup(this);
    //initialize actions
    bold = addChoice("Bold", "&Bold");
    underline = addChoice("Underline", "&Underline");
    italics = addChoice("Italics", "&Italics");
    save = addChoice("Save", "&Save");
    undo = addChoice("Undo", "&Undo");
    //declare and initialize a menu
    QMenu* editMenu = new QMenu("&Edit", this);
    //add items to the menu    
    editMenu->addAction(bold);
    editMenu->addAction(underline);
    editMenu->addAction(italics);
    QMenu *otherMenu = new QMenu("&Other", this);
    otherMenu->addAction(save);
    otherMenu->addAction(undo);
     //declare and initialize a toolbar
    QToolBar* toolbar = new QToolBar("Edit", this);
    //add all actions to the toolbar
    toolbar->addActions(actionGroup->actions());
    //add the menu to the QMainWindow
    QMainWindow::menuBar()->addMenu(editMenu);
    QMainWindow::menuBar()->addMenu(otherMenu);
    //add the text edit to the QMainWindow
    QMainWindow::setCentralWidget(textEdit);
    //add toolbar in the top area
    QMainWindow::addToolBar(Qt::TopToolBarArea, toolbar);
    //connect events generated by the action group to the event handling function.
    QObject::connect(actionGroup, SIGNAL(triggered(QAction*)), this, SLOT(actionEvent(QAction*)));
    //resize the widget
    QWidget::move(300, 300);
    QWidget::resize(200, 200);

    undos = 5;
}

MyNotepad::~MyNotepad(){
    while(mementos.size()>=1){
        TextAndStyleMemento *t = mementos.pop();
        delete t;
    }
}

//creates, adds and returns a QAction to the action group
QAction* MyNotepad::addChoice(QString name, QString text) {
    QAction* retval = new QAction(text, this);
    retval->setObjectName(name);
    actionGroup->addAction(retval);
    return retval;
}

// handles events generated by the menu or toolbar
void MyNotepad::actionEvent(QAction* act) {
   QString name = act->objectName();
   setStyle(name);
}

void MyNotepad::setBold(QTextCursor& tc, QTextCharFormat format){
    tc.setCharFormat(format);
}

void MyNotepad::setUnderline(QTextCursor& tc, QTextCharFormat format){
    tc.setCharFormat(format);
}

void MyNotepad::setItalics(QTextCursor& tc, QTextCharFormat format){
    tc.setCharFormat(format);
}

void MyNotepad::createAndStoreMemento(){
    QString tex = textEdit->toPlainText();
    QString sty;
    if(textEdit->fontWeight()==75)
        sty = "Bold";
    if(textEdit->fontItalic()==true)
        sty = "Italics";
    if(textEdit->fontUnderline()==true)
        sty = "Underline";

    TextAndStyle tas(tex, sty);
    TextAndStyleMemento* tasm = tas.createMemento();
    mementos.append(tasm);

    if(mementos.size() > undos){
        TextAndStyleMemento *m = mementos.at(0);
        mementos.remove(0);
        delete m;
    }
}

void MyNotepad::restore(){
    if(!mementos.isEmpty()){
        TextAndStyle tas;
        TextAndStyleMemento *tasm = mementos.pop();
        tas.setMemento(tasm);        
        textEdit->setPlainText(tas.getText());
        QString s = tas.getStyle();
        setStyle(s);
        delete tasm;
    }
}

void MyNotepad::setStyle(QString st){
    QTextCursor tc = textEdit->textCursor();
    tc.movePosition(QTextCursor::Start);
    tc.select(QTextCursor::Document);
    QTextCharFormat f;

    if (st == "Bold") {
        f.setFontWeight(QFont::Bold);
        setBold(tc, f);
     }
    if (st == "Underline") {
        f.setFontUnderline(true);
        setUnderline(tc, f);
    }
    if (st == "Italics") {
        f.setFontItalic(true);
        setItalics(tc,f);
    }
    if (st == "Save"){
        createAndStoreMemento();
    }
    if (st == "Undo"){
        restore();
    }
}
