#include "counterview.h"
#include <QGridLayout>
#include <QDockWidget>
#include <QDebug>

QDockWidget* CounterView::returnDockWidget(QWidget* w, QString title){
    QDockWidget *threadDock = new QDockWidget;
    threadDock->setWindowTitle(title);
    threadDock->setWidget(w);
    return threadDock;
}

CounterView::CounterView(int argc, char* argv[]):QApplication(argc, argv){

    result = new QTextEdit();
    result->setReadOnly(true);

  // {{{ Left Docking Area
    cv1 = new CounterViewWidget("Thread 1");
    QDockWidget* dw1 = returnDockWidget(cv1, "Thread 1");
    mainWindow.addDockWidget(Qt::LeftDockWidgetArea, dw1);
  // }}}Left Docking Area

  // {{{ Right Docking Area
    cv2 = new CounterViewWidget("Thread 2");
    QDockWidget* dw2 = returnDockWidget(cv2, "Thread 2");
    mainWindow.addDockWidget(Qt::RightDockWidgetArea, dw2);
  // }}}Right Docking Area

  // {{{ Bottom Docking Area
  QDockWidget *resultDock = new QDockWidget;
  resultDock->setWindowTitle("Result");
  resultDock->setWidget(result);
  mainWindow.addDockWidget(Qt::BottomDockWidgetArea, resultDock);
  // }}}Bottom Docking Area

  c1 = new Counter();  
  c2 = new Counter();
  t1 = new QThread();
  t2 = new QThread();
  c1->moveToThread(t1);
  c2->moveToThread(t2);

  //{{ Connect signals and slots
  connect(cv1, SIGNAL(valuesEntered(int,int,int,int,QString)), SLOT(startThread(int,int,int,int,QString)));
  connect(cv2, SIGNAL(valuesEntered(int,int,int,int,QString)), SLOT(startThread(int,int,int,int,QString)));
  connect(c1, SIGNAL(countValue(int)), this, SLOT(update(int)));
  connect(c2, SIGNAL(countValue(int)), this, SLOT(update(int)));
  connect(t1, SIGNAL(started()), c1, SLOT(count()));
  connect(t2, SIGNAL(started()), c2, SLOT(count()));
  connect(c1, SIGNAL(finished()), t1, SLOT(quit()));
  connect(c2, SIGNAL(finished()), t2, SLOT(quit()));
  //}} Connect signals and slots

  mainWindow.show();  
}

CounterView::~CounterView(){
    t1->quit();
    t2->quit();
    t1->deleteLater();
    t2->deleteLater();
    delete c1;
    delete c2;
}

void CounterView::startThread(int s, int e, int i, int d, QString t){

    Counter *c;
    QThread *th;

    if (t == "Thread 1"){
        c = c1;
        th = t1;        
    }
    if(t == "Thread 2"){
        c = c2;
        th = t2;        
    }
    if(c != 0){
        c->setStart(s);
        c->setEnd(e);
        c->setIncrement(i);
        c->setDelay(d);
        th->start();
    }
}

void CounterView::update(int i){    
     result->append(QString::number(i,10));
}
